"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("@firebase/util");
var app_1 = require("./app");
var utils_1 = require("../utils");
exports.DEFAULT_ENTRY_NAME = '[DEFAULT]';
var contains = function (obj, key) {
    return Object.prototype.hasOwnProperty.call(obj, key);
};
/**
 * Return a firebase namespace object.
 *
 * In production, this will be called exactly once and the result
 * assigned to the 'firebase' global.  It may be called multiple times
 * in unit tests.
 */
// tslint:disable-next-line
function createFirebaseNamespace() {
    var apps_ = {};
    var factories = {};
    var appHooks = {};
    // A namespace is a plain JavaScript Object.
    var namespace = {
        // Hack to prevent Babel from modifying the object returned
        // as the firebase namespace.
        __esModule: true,
        initializeApp: initializeApp,
        app: app,
        apps: null,
        Promise: Promise,
        SDK_VERSION: 'ts-mock-firebase',
        INTERNAL: {
            registerService: registerService,
            createFirebaseNamespace: createFirebaseNamespace,
            extendNamespace: extendNamespace,
            createSubscribe: util_1.createSubscribe,
            ErrorFactory: util_1.ErrorFactory,
            removeApp: removeApp,
            factories: factories,
            useAsService: useAsService,
            Promise: Promise,
            deepExtend: util_1.deepExtend,
        },
    };
    // Inject a circular default export to allow Babel users who were previously
    // using:
    //
    //   import firebase from 'firebase';
    //   which becomes: var firebase = require('firebase').default;
    //
    // instead of
    //
    //   import * as firebase from 'firebase';
    //   which becomes: var firebase = require('firebase');
    utils_1.patchProperty(namespace, 'default', namespace);
    // firebase.apps is a read-only getter.
    Object.defineProperty(namespace, 'apps', {
        get: getApps,
    });
    /**
     * Called by App.delete() - but before any services associated with the App
     * are deleted.
     */
    function removeApp(name) {
        var appToBeRemoved = apps_[name];
        callAppHooks(appToBeRemoved, 'delete');
        delete apps_[name];
    }
    /**
     * Get the App object for a given name (or DEFAULT).
     */
    function app(name) {
        var nameOfApp = name || exports.DEFAULT_ENTRY_NAME;
        if (!contains(apps_, nameOfApp)) {
            error('no-app', { name: nameOfApp });
        }
        return apps_[nameOfApp];
    }
    utils_1.patchProperty(app, 'App', app_1.MockFirebaseAppImpl);
    /**
     * Create a new App instance (name must be unique).
     */
    function initializeApp(options, configOrName) {
        var config = typeof configOrName === 'object' ? configOrName : { name: configOrName };
        if (config.name === undefined) {
            config.name = exports.DEFAULT_ENTRY_NAME;
        }
        var name = config.name;
        if (typeof name !== 'string' || !name) {
            // tslint:disable-next-line
            error('bad-app-name', { name: name + '' });
        }
        if (contains(apps_, name)) {
            // tslint:disable-next-line
            error('duplicate-app', { name: name });
        }
        var newApp = new app_1.MockFirebaseAppImpl(options, config, namespace); // TODO resolve this
        apps_[name] = newApp;
        // callAppHooks(newApp, 'create');
        return newApp;
    }
    /*
     * Return an array of all the non-deleted FirebaseApps.
     */
    function getApps() {
        // Make a copy so caller cannot mutate the apps list.
        return Object.keys(apps_).map(function (name) { return apps_[name]; });
    }
    /*
     * Register a Firebase Service.
     *
     * firebase.INTERNAL.registerService()
     *
     * TODO: Implement serviceProperties.
     */
    function registerService(name, createService, serviceProperties, appHook, 
    // tslint:disable-next-line
    allowMultipleInstances) {
        var _this = this;
        // Cannot re-register a service that already exists
        if (factories[name]) {
            // tslint:disable-next-line
            error('duplicate-service', { name: name });
        }
        // Capture the service factory for later service instantiation
        factories[name] = createService;
        // Capture the appHook, if passed
        if (appHook) {
            appHooks[name] = appHook;
            // Run the **new** app hook on all existing apps
            getApps().forEach(function (anApp) {
                appHook('create', anApp);
            });
        }
        // The Service namespace is an accessor function ...
        var serviceNamespace = function (appArg) {
            if (appArg === void 0) { appArg = app(); }
            if (typeof appArg[name] !== 'function') {
                // Invalid argument.
                // This happens in the following case: firebase.storage('gs:/')
                // tslint:disable-next-line
                error('invalid-app-argument', { name: name });
            }
            // Forward service instance lookup to the FirebaseApp.
            return appArg[name]();
        };
        // ... and a container for service-level properties.
        if (serviceProperties !== undefined) {
            util_1.deepExtend(serviceNamespace, serviceProperties);
        }
        // tslint:disable-next-line
        // Monkey-patch the serviceNamespace onto the firebase namespace
        namespace[name] = serviceNamespace;
        // Patch the FirebaseAppImpl prototype
        app_1.MockFirebaseAppImpl.prototype[name] = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            // tslint:disable-next-line
            var self = _this;
            var serviceFxn = self._getService.bind(self, name);
            return serviceFxn.apply(self, allowMultipleInstances ? args : []);
        };
        return serviceNamespace;
    }
    /**
     * Patch the top-level firebase namespace with additional properties.
     *
     * firebase.INTERNAL.extendNamespace()
     */
    function extendNamespace(props) {
        util_1.deepExtend(namespace, props);
    }
    function callAppHooks(hookedApp, eventName) {
        Object.keys(factories).forEach(function (serviceName) {
            // Ignore virtual services
            var factoryName = useAsService(hookedApp, serviceName);
            if (factoryName === null) {
                return;
            }
            if (appHooks[factoryName]) {
                appHooks[factoryName](eventName, hookedApp);
            }
        });
    }
    // Map the requested service to a registered service name
    // (used to map auth to serverAuth service when needed).
    function useAsService(targetApp, name) {
        if (name === 'serverAuth') {
            return null;
        }
        // tslint:disable-next-line
        // const useService = name
        // tslint:disable-next-line
        // const options = targetApp.options
        return name;
    }
    return namespace; // TODO check this out
}
exports.createFirebaseNamespace = createFirebaseNamespace;
// TypeScript does not support non-string indexes!
// let errors: {[code: AppError: string} = {
var errors = {
    'no-app': "No Firebase App '{$name}' has been created - " + 'call Firebase App.initializeApp()',
    'bad-app-name': "Illegal App name: '{$name}",
    'duplicate-app': "Firebase App named '{$name}' already exists",
    'app-deleted': "Firebase App named '{$name}' already deleted",
    'duplicate-service': "Firebase service named '{$name}' already registered",
    'sa-not-supported': 'Initializing the Firebase SDK with a service ' +
        'account is only allowed in a Node.js environment. On client ' +
        'devices, you should instead initialize the SDK with an api key and ' +
        'auth domain',
    'invalid-app-argument': 'firebase.{$name}() takes either no argument or a ' + 'Firebase App instance.',
};
var appErrors = new util_1.ErrorFactory('app', 'Firebase', errors);
function error(code, args) {
    throw appErrors.create(code);
}
exports.error = error;
__export(require("./app"));
__export(require("../firestore"));
__export(require("../auth"));
__export(require("../utils"));
// export const firebaseNamespace = createFirebaseNamespace();
// export default firebaseNamespace;
//# sourceMappingURL=index.js.map