import { ActionCodeInfo, ActionCodeSettings, ApplicationVerifier, AuthCredential, AuthProvider, AuthSettings, ConfirmationResult, Error, FirebaseAuth, Persistence, User, UserCredential } from '@firebase/auth-types';
import { Observer, Unsubscribe } from '@firebase/util';
import { MockFirebaseAppImpl } from '../app';
import MockUser from './MockUser';
export default class MockFirebaseAuth implements FirebaseAuth {
    app: MockFirebaseAppImpl;
    get currentUser(): MockUser | null;
    set currentUser(user: MockUser | null);
    get languageCode(): string | null;
    get settings(): AuthSettings;
    tenantId: string | null;
    static Persistence: {
        LOCAL: Persistence;
        NONE: Persistence;
        SESSION: Persistence;
    };
    private _user;
    constructor(app: MockFirebaseAppImpl);
    applyActionCode: (code: string) => Promise<void>;
    checkActionCode: (code: string) => Promise<ActionCodeInfo>;
    confirmPasswordReset: (code: string, newPassword: string) => Promise<void>;
    createUserWithEmailAndPassword: (email: string, password: string) => Promise<UserCredential>;
    createUserAndRetrieveDataWithEmailAndPassword: (email: string, password: string) => Promise<UserCredential>;
    fetchProvidersForEmail: (email: string) => Promise<string[]>;
    fetchSignInMethodsForEmail: (email: string) => Promise<string[]>;
    isSignInWithEmailLink: (emailLink: string) => boolean;
    getRedirectResult: () => Promise<UserCredential>;
    onAuthStateChanged: (nextOrObserver: Observer<any> | ((a: User | null) => any), error?: ((a: Error) => any) | undefined, completed?: Unsubscribe | undefined) => Unsubscribe;
    onIdTokenChanged: (nextOrObserver: Observer<any> | ((a: User | null) => any), error?: ((a: Error) => any) | undefined, completed?: Unsubscribe | undefined) => Unsubscribe;
    sendSignInLinkToEmail: (email: string, actionCodeSettings: ActionCodeSettings) => Promise<void>;
    sendPasswordResetEmail: (email: string, actionCodeSettings?: ActionCodeSettings | null | undefined) => Promise<void>;
    setPersistence: (persistence: string) => Promise<void>;
    signInAndRetrieveDataWithCredential: (credential: AuthCredential) => Promise<UserCredential>;
    signInAnonymously: () => Promise<UserCredential>;
    signInAnonymouslyAndRetrieveData: () => Promise<UserCredential>;
    signInWithCredential: (credential: AuthCredential) => Promise<UserCredential>;
    signInWithCustomToken: (token: string) => Promise<UserCredential>;
    signInAndRetrieveDataWithCustomToken: (token: string) => Promise<UserCredential>;
    signInWithEmailAndPassword: (email: string, password: string) => Promise<UserCredential>;
    signInAndRetrieveDataWithEmailAndPassword: (email: string, password: string) => Promise<UserCredential>;
    signInWithEmailLink: (email: string, emailLink?: string | undefined) => Promise<UserCredential>;
    signInWithPhoneNumber: (phoneNumber: string, applicationVerifier: ApplicationVerifier) => Promise<ConfirmationResult>;
    signInWithPopup: (provider: AuthProvider) => Promise<UserCredential>;
    signInWithRedirect: (provider: AuthProvider) => Promise<void>;
    signOut: () => Promise<void>;
    updateCurrentUser: (user: User | null) => Promise<void>;
    useDeviceLanguage: () => void;
    verifyPasswordResetCode: (code: string) => Promise<string>;
}
