import { Blob } from '@firebase/firestore-types';
/**
 * An immutable object representing an array of bytes.
 */
export declare class MockBlob implements Blob {
    /**
     * Creates a new Blob from the given Base64 string, converting it to
     * bytes.
     */
    static fromBase64String(base64: string): MockBlob;
    /**
     * Creates a new Blob from the given Uint8Array.
     */
    static fromUint8Array(array: Uint8Array): MockBlob;
    private _binaryString;
    private constructor();
    /**
     * Returns the bytes of this Blob as a Base64-encoded string.
     */
    toBase64(): string;
    /**
     * Returns the bytes of this Blob in a new Uint8Array.
     */
    toUint8Array(): Uint8Array;
    /**
     * Returns true if this `Blob` is equal to the provided one.
     *
     * @param other The `Blob` to compare against.
     * @return true if this `Blob` is equal to the provided one.
     */
    isEqual(other: MockBlob): boolean;
    toString(): string;
    /**
     * Actually private to JS consumers of our API, so this function is prefixed
     * with an underscore.
     */
    _compareTo(other: MockBlob): number;
}
