import { FieldPath } from '@firebase/firestore-types';
/**
 * A FieldPath refers to a field in a document. The path may consist of a
 * single field name (referring to a top-level field in the document), or a
 * list of field names (referring to a nested field in the document).
 */
export declare class MockFieldPath implements FieldPath {
    get path(): string;
    get fieldNames(): string[];
    /**
     * Returns a special sentinel FieldPath to refer to the ID of a document.
     * It can be used in queries to sort or filter by the document ID.
     */
    static get documentId(): FieldPath;
    private _path;
    private _fieldNames;
    /**
     * Creates a FieldPath from the provided field names. If more than one field
     * name is provided, the path will point to a nested field in a document.
     *
     * @param fieldNames A list of field names.
     */
    constructor(...fieldNames: string[]);
    /**
     * Returns true if this `FieldPath` is equal to the provided one.
     *
     * @param other The `FieldPath` to compare against.
     * @return true if this `FieldPath` is equal to the provided one.
     */
    isEqual: (other: FieldPath) => boolean;
}
/**
 * Special sentinel to be used to mark document id
 */
export declare const documentIdSentinel: MockFieldPath;
/**
 * Create a FieldPath object presentation from dot delimeted string presentation
 *
 * @param fieldPath Field path in a dot delimeted presentation
 * @returns FieldPath object representing the given field path in separate items
 */
export declare function createFieldPathFromString(fieldPath: string): MockFieldPath;
