"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * A FieldPath refers to a field in a document. The path may consist of a
 * single field name (referring to a top-level field in the document), or a
 * list of field names (referring to a nested field in the document).
 */
var MockFieldPath = /** @class */ (function () {
    /**
     * Creates a FieldPath from the provided field names. If more than one field
     * name is provided, the path will point to a nested field in a document.
     *
     * @param fieldNames A list of field names.
     */
    function MockFieldPath() {
        var _this = this;
        var fieldNames = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            fieldNames[_i] = arguments[_i];
        }
        /**
         * Returns true if this `FieldPath` is equal to the provided one.
         *
         * @param other The `FieldPath` to compare against.
         * @return true if this `FieldPath` is equal to the provided one.
         */
        this.isEqual = function (other) {
            return other.path === _this.path;
        };
        this._fieldNames = fieldNames;
        this._path = fieldNames.reduce(function (path, value) { return (path ? path + "." + value : value); }, '');
    }
    Object.defineProperty(MockFieldPath.prototype, "path", {
        get: function () {
            return this._path;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockFieldPath.prototype, "fieldNames", {
        get: function () {
            return this._fieldNames;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockFieldPath, "documentId", {
        /**
         * Returns a special sentinel FieldPath to refer to the ID of a document.
         * It can be used in queries to sort or filter by the document ID.
         */
        get: function () {
            // tslint:disable-next-line: no-use-before-declare
            return exports.documentIdSentinel;
        },
        enumerable: true,
        configurable: true
    });
    return MockFieldPath;
}());
exports.MockFieldPath = MockFieldPath;
/**
 * Special sentinel to be used to mark document id
 */
exports.documentIdSentinel = new MockFieldPath('DOC_ID_SENTINEL');
/**
 * Create a FieldPath object presentation from dot delimeted string presentation
 *
 * @param fieldPath Field path in a dot delimeted presentation
 * @returns FieldPath object representing the given field path in separate items
 */
function createFieldPathFromString(fieldPath) {
    var fieldNames = fieldPath.split('.');
    return new (MockFieldPath.bind.apply(MockFieldPath, __spreadArrays([void 0], fieldNames)))();
}
exports.createFieldPathFromString = createFieldPathFromString;
//# sourceMappingURL=MockFieldPath.js.map