import { DocumentChange, Query, QueryDocumentSnapshot, QuerySnapshot, SnapshotListenOptions, SnapshotMetadata, DocumentData } from '@firebase/firestore-types';
/**
 * A `QuerySnapshot` contains zero or more `DocumentSnapshot` objects
 * representing the results of a query. The documents can be accessed as an
 * array via the `docs` property or enumerated using the `forEach` method. The
 * number of documents can be determined via the `empty` and `size`
 * properties.
 */
export default class MockQuerySnapshot<T = DocumentData> implements QuerySnapshot<DocumentData> {
    query: Query;
    private _docs;
    private _docChanges;
    /**
     * An array of all the documents in the QuerySnapshot.
     */
    get docs(): QueryDocumentSnapshot<T>[];
    /** The number of documents in the QuerySnapshot. */
    get size(): number;
    /** True if there are no documents in the QuerySnapshot. */
    get empty(): boolean;
    /**
     * The query on which you called `get` or `onSnapshot` in order to get this
     * `QuerySnapshot`.
     */
    /**
     * Metadata about this snapshot, concerning its source and if it has local
     * modifications.
     */
    metadata: SnapshotMetadata;
    constructor(query: Query, _docs: QueryDocumentSnapshot[], _docChanges: DocumentChange[]);
    /**
     * Returns an array of the documents changes since the last snapshot. If this
     * is the first snapshot, all documents will be in the list as added changes.
     *
     * @param options `SnapshotListenOptions` that control whether metadata-only
     * changes (i.e. only `DocumentSnapshot.metadata` changed) should trigger
     * snapshot events.
     */
    docChanges: (options?: SnapshotListenOptions | undefined) => DocumentChange<T>[];
    /**
     * Enumerates all of the documents in the QuerySnapshot.
     *
     * @param callback A callback to be called with a `QueryDocumentSnapshot` for
     * each document in the snapshot.
     * @param thisArg The `this` binding for the callback.
     */
    forEach: (callback: (result: QueryDocumentSnapshot<T>) => void, thisArg?: any) => void;
    /**
     * Returns true if this `QuerySnapshot` is equal to the provided one.
     *
     * @param other The `QuerySnapshot` to compare against.
     * @return true if this `QuerySnapshot` is equal to the provided one.
     */
    isEqual: (other: QuerySnapshot<DocumentData>) => boolean;
}
