"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * A `QuerySnapshot` contains zero or more `DocumentSnapshot` objects
 * representing the results of a query. The documents can be accessed as an
 * array via the `docs` property or enumerated using the `forEach` method. The
 * number of documents can be determined via the `empty` and `size`
 * properties.
 */
var MockQuerySnapshot = /** @class */ (function () {
    function MockQuerySnapshot(query, _docs, _docChanges) {
        var _this = this;
        this.query = query;
        this._docs = _docs;
        this._docChanges = _docChanges;
        /**
         * The query on which you called `get` or `onSnapshot` in order to get this
         * `QuerySnapshot`.
         */
        // query: Query;
        /**
         * Metadata about this snapshot, concerning its source and if it has local
         * modifications.
         */
        this.metadata = {
            hasPendingWrites: false,
            fromCache: false,
            isEqual: function (other) {
                return other.hasPendingWrites === false && other.fromCache === false;
            },
        };
        /**
         * Returns an array of the documents changes since the last snapshot. If this
         * is the first snapshot, all documents will be in the list as added changes.
         *
         * @param options `SnapshotListenOptions` that control whether metadata-only
         * changes (i.e. only `DocumentSnapshot.metadata` changed) should trigger
         * snapshot events.
         */
        this.docChanges = function (options) {
            return _this._docChanges;
        };
        /**
         * Enumerates all of the documents in the QuerySnapshot.
         *
         * @param callback A callback to be called with a `QueryDocumentSnapshot` for
         * each document in the snapshot.
         * @param thisArg The `this` binding for the callback.
         */
        this.forEach = function (callback, thisArg) {
            // Todo this binding
            for (var _i = 0, _a = _this._docs; _i < _a.length; _i++) {
                var documentSnapshot = _a[_i];
                callback(documentSnapshot);
            }
        };
        /**
         * Returns true if this `QuerySnapshot` is equal to the provided one.
         *
         * @param other The `QuerySnapshot` to compare against.
         * @return true if this `QuerySnapshot` is equal to the provided one.
         */
        this.isEqual = function (other) {
            return _this === other;
        };
    }
    Object.defineProperty(MockQuerySnapshot.prototype, "docs", {
        /**
         * An array of all the documents in the QuerySnapshot.
         */
        get: function () {
            return this._docs.slice();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockQuerySnapshot.prototype, "size", {
        /** The number of documents in the QuerySnapshot. */
        get: function () {
            return this._docs.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockQuerySnapshot.prototype, "empty", {
        /** True if there are no documents in the QuerySnapshot. */
        get: function () {
            return this._docs.length === 0;
        },
        enumerable: true,
        configurable: true
    });
    return MockQuerySnapshot;
}());
exports.default = MockQuerySnapshot;
//# sourceMappingURL=MockQuerySnapshot.js.map