import { MockFirebaseFirestore } from '@firebase/app-types';
import { DocumentChange, DocumentData, FieldPath, SetOptions, Transaction, UpdateData, DocumentReference } from '@firebase/firestore-types';
import MockDocumentReference from './MockDocumentReference';
import MockDocumentSnapshot from './MockDocumentSnapshot';
import MockQueryDocumentSnapshot from './MockQueryDocumentSnapshot';
export interface MockDocumentChange<T = DocumentData> extends DocumentChange<DocumentData> {
    readonly doc: MockQueryDocumentSnapshot<T>;
}
/**
 * A reference to a transaction.
 * The `Transaction` object passed to a transaction's updateFunction provides
 * the methods to read and write data within the transaction context. See
 * `Firestore.runTransaction()`.
 */
export default class MockTransaction implements Transaction {
    firestore: MockFirebaseFirestore;
    private transactionData;
    private transactionOperation;
    /**
     * True if any of set, update and delete -methods have been called. Firestore do not allow any reading
     * of data after modifications are done. This field is used internally to indicate if these operations
     * have been done.
     *
     * @private
     * @memberof MockTransaction
     */
    private modified;
    constructor(firestore: MockFirebaseFirestore);
    /**
     * Reads the document referenced by the provided `DocumentReference.`
     *
     * @param documentRef A reference to the document to be read.
     * @return A DocumentSnapshot for the read data.
     */
    get: <T = DocumentData>(documentRef: DocumentReference<T>) => Promise<MockDocumentSnapshot<any>>;
    /**
     * Writes to the document referred to by the provided `DocumentReference`.
     * If the document does not exist yet, it will be created. If you pass
     * `SetOptions`, the provided data can be merged into the existing document.
     *
     * @param documentRef A reference to the document to be set.
     * @param data An object of the fields and values for the document.
     * @param options An object to configure the set behavior.
     * @return This `Transaction` instance. Used for chaining method calls.
     */
    set: <T = DocumentData>(documentRef: DocumentReference<T>, data: T, options?: SetOptions | undefined) => MockTransaction;
    /**
     * Updates fields in the document referred to by the provided
     * `DocumentReference`. The update will fail if applied to a document that
     * does not exist.
     *
     * @param documentRef A reference to the document to be updated.
     * @param data An object containing the fields and values with which to
     * @param field The first field to update.
     * @param value The first value.
     * @param moreFieldsAndValues Additional key/value pairs.
     * update the document. Fields can contain dots to reference nested fields
     * within the document.
     * @return This `Transaction` instance. Used for chaining method calls.
     */
    update: (documentRef: MockDocumentReference<DocumentData>, dataOrField: string | UpdateData | FieldPath, ...moreFieldsAndValues: any[]) => MockTransaction;
    /**
     * Deletes the document referred to by the provided `DocumentReference`.
     *
     * @param documentRef A reference to the document to be deleted.
     * @return This `Transaction` instance. Used for chaining method calls.
     */
    delete: (documentRef: MockDocumentReference<DocumentData>) => Transaction;
    commit: () => Promise<void>;
    rollback: () => void;
    private updateFieldsFromArgs;
    private updateSingleFieldPathValue;
}
