"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var MockDocumentSnapshot_1 = require("../MockDocumentSnapshot");
var MockFieldPath_1 = require("../MockFieldPath");
describe('MockDocumentSnapshot', function () {
    describe('get', function () {
        it('Will retrieve the field value specified by the field name.', function () {
            var data = {
                text: 'value',
                text2: 'some',
            };
            var snapshot = new MockDocumentSnapshot_1.default({}, data);
            expect(snapshot.get('text')).toBe('value');
        });
        it('Will return undefined if the field does not exist.', function () {
            var data = {
                text: 'value',
                text2: 'some',
            };
            var snapshot = new MockDocumentSnapshot_1.default({}, data);
            expect(snapshot.get('text3')).toBeUndefined();
        });
        it('Will retrieve the field value specified by a FieldPath.', function () {
            var data = {
                subset: {
                    text: 'value',
                },
                text2: 'some',
            };
            var snapshot = new MockDocumentSnapshot_1.default({}, data);
            var path = new MockFieldPath_1.MockFieldPath('subset', 'text');
            expect(snapshot.get(path)).toBe('value');
        });
        it('Will return undefined when specified field by a FieldPath does not exist.', function () {
            var data = {
                subset: {
                    text: 'value',
                },
                text2: 'some',
            };
            var snapshot = new MockDocumentSnapshot_1.default({}, data);
            var path = new MockFieldPath_1.MockFieldPath('subset', 'cat');
            expect(snapshot.get(path)).toBeUndefined();
        });
    });
});
//# sourceMappingURL=MockDocumentSnapshot.spec.js.map