"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var MockTimestamp_1 = require("../MockTimestamp");
describe('MockTimestamp', function () {
    it('will create a timestamp from a date', function () {
        var date = new Date('2019-03-11 10:37');
        var timestamp = MockTimestamp_1.MockTimestamp.fromDate(date);
        var milliseconds = date.getTime();
        var seconds = Math.floor(milliseconds / 1000);
        var nanoseconds = (milliseconds - seconds * 1000) * 1000000;
        expect(timestamp.seconds).toBe(seconds);
        expect(timestamp.nanoseconds).toBe(nanoseconds);
    });
    it('will create a timestamp from now', function () {
        var date = new Date();
        var timestamp = MockTimestamp_1.MockTimestamp.now();
        expect(timestamp.seconds * 1000 + timestamp.nanoseconds / 1000000 - date.getTime()).toBeLessThan(200);
    });
    it('will create a timestamp from milliseconds', function () {
        var date = 5500.5;
        var timestamp = MockTimestamp_1.MockTimestamp.fromMillis(date);
        expect(timestamp.seconds).toBe(5);
        expect(timestamp.nanoseconds).toBe(500500000);
    });
    it('will return value back to date', function () {
        var date = new Date('2019-03-11 12:37');
        var timestamp = MockTimestamp_1.MockTimestamp.fromDate(date);
        expect(timestamp.toDate()).toEqual(date);
    });
    it('will return value back to milliseconds', function () {
        var date = new Date('2019-03-11 12:37');
        var timestamp = MockTimestamp_1.MockTimestamp.fromMillis(date.getTime());
        expect(timestamp.toMillis()).toEqual(date.getTime());
    });
    describe('Equality', function () {
        it('will match equal timestamps', function () {
            var date = new Date('2019-03-11 12:37');
            var date2 = new Date('2019-03-11 12:37');
            var timestamp = MockTimestamp_1.MockTimestamp.fromMillis(date.getTime());
            var timestamp2 = MockTimestamp_1.MockTimestamp.fromMillis(date2.getTime());
            expect(timestamp.isEqual(timestamp2)).toBe(true);
        });
        it('will unmatch different timestamps', function () {
            var date = new Date('2019-03-11 12:37');
            var date2 = new Date('2019-01-13 10:37');
            var timestamp = MockTimestamp_1.MockTimestamp.fromMillis(date.getTime());
            var timestamp2 = MockTimestamp_1.MockTimestamp.fromMillis(date2.getTime());
            expect(timestamp.isEqual(timestamp2)).toBe(false);
        });
    });
});
//# sourceMappingURL=MockTimestamp.spec.js.map