"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Callback handler used for basic operation in onSnapshot -handlers.
 *
 * @export
 * @class MockListener
 * @template S Type of snapshot object
 */
var MockCallbackHandler = /** @class */ (function () {
    function MockCallbackHandler() {
        var _this = this;
        this._callbacks = [];
        /**
         * Add a new snapshot callback
         * @param {<S>(snapshot: S) => void} callback callback function
         */
        this.add = function (callback) {
            if (_this._callbacks.indexOf(callback) < 0) {
                _this._callbacks.push(callback);
            }
        };
        this.remove = function (callback) {
            var index = _this._callbacks.indexOf(callback);
            if (index >= 0) {
                _this._callbacks.splice(index, 1);
            }
        };
        this.fire = function (snapshot, callbacks) {
            if (callbacks === void 0) { callbacks = _this._callbacks; }
            for (var _i = 0, callbacks_1 = callbacks; _i < callbacks_1.length; _i++) {
                var callback = callbacks_1[_i];
                callback(snapshot);
            }
        };
        this.load = function (callbacks) {
            _this._callbacks = callbacks.slice();
        };
        this.reset = function () {
            _this._callbacks = [];
        };
    }
    Object.defineProperty(MockCallbackHandler.prototype, "list", {
        get: function () {
            return this._callbacks.slice();
        },
        enumerable: true,
        configurable: true
    });
    return MockCallbackHandler;
}());
exports.default = MockCallbackHandler;
//# sourceMappingURL=CallbackHandler.js.map