"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var CallbackHandler_1 = require("../CallbackHandler");
describe('CallbackHandler', function () {
    it('will register callback', function () {
        var handler = new CallbackHandler_1.default();
        var callback = function (snapshot) {
            console.log(snapshot);
        };
        handler.add(callback);
        expect(handler.list.length).toBe(1);
        expect(handler.list).toContain(callback);
    });
    it('will not register same callback twice', function () {
        var handler = new CallbackHandler_1.default();
        var callback = function (snapshot) {
            console.log(snapshot);
        };
        handler.add(callback);
        handler.add(callback);
        expect(handler.list.length).toBe(1);
        expect(handler.list).toContain(callback);
    });
    it('will remove registerered from handler', function () {
        var handler = new CallbackHandler_1.default();
        var callback = function (snapshot) {
            console.log(snapshot);
        };
        handler.add(callback);
        handler.remove(callback);
        expect(handler.list.length).toBe(0);
    });
    it('reset will remove all callbacks', function () {
        var handler = new CallbackHandler_1.default();
        var callback = function (snapshot) {
            console.log(snapshot);
        };
        handler.add(callback);
        handler.reset();
        expect(handler.list.length).toBe(0);
    });
    it('will fire each callback once', function () {
        var handler = new CallbackHandler_1.default();
        var callbackA = jest.fn(function (s) {
            //console.log(s);
        });
        var callbackB = jest.fn(function (s) {
            //console.log(s);
        });
        handler.add(callbackA);
        handler.add(callbackB);
        var snapshot = {};
        handler.fire(snapshot);
        expect(callbackA.mock.calls.length).toBe(1);
        expect(callbackB.mock.calls.length).toBe(1);
        expect(callbackA.mock.calls[0][0]).toBe(snapshot);
        expect(callbackB.mock.calls[0][0]).toBe(snapshot);
    });
});
//# sourceMappingURL=CallbackHandler.spec.js.map