import { MockFirebaseFirestore } from '@firebase/app-types';
import { FieldPath, DocumentData } from '@firebase/firestore-types';
import { MockCollectionReference } from '../MockCollectionReference';
import MockDocumentReference from '../MockDocumentReference';
export declare class MockFirebaseValidationError extends Error {
}
/**
 * Generate an unique document id
 */
export declare function generateDocumentId(): string;
/**
 * Validates document
 * Must be valid UTF-8 characters
 * Must be no longer than 1,500 bytes
 * Cannot contain a forward slash (/)
 * Cannot solely consist of a single period (.) or double periods (..)
 * Cannot match the regular expression __.*__
 *
 * @param path Constraints on document IDs
 */
export declare function isValidDocumentId(id: string): boolean;
/**
 * Validate Firestore document path reference string
 * @param path
 */
export declare function isValidDocumentReference(path: string): boolean;
/**
 * Validate Firestore document path reference string
 * @param path
 */
export declare function isValidCollectionReference(path: string): boolean;
/**
 * Validated if field fulfills requirements set to Firestore field names:
 *
 * Must enclose each field name in backticks unless the field name meets the following requirements:
 * - The field name contains only the characters a-z, A-Z, 0-9, and underscore (_)
 * - The field name does not start with 0-9
 *
 * Maximum size of a field name is 1,500 bytes
 *
 * @see https://firebase.google.com/docs/firestore/quotas#limits
 * @param name
 */
export declare function isValidFirestoreFieldName(name: string): boolean;
/**
 * Validated if field fulfills requirements set to Firestore field name paths:
 *
 * Constraints on field paths:
 * - Must separate field names with a single period (.)
 *
 * Must enclose each field name in backticks unless the field name meets the following requirements:
 * - The field name contains only the characters a-z, A-Z, 0-9, and underscore (_)
 * - The field name does not start with 0-9
 *
 * Maximum size of a field path	is 1,500 bytes
 *
 * @see https://firebase.google.com/docs/firestore/quotas#limits
 * @param name
 */
export declare function isValidFirestoreFieldPath(name: string): boolean;
export declare function isValidUnicodeCharacter(c: string): boolean;
export declare function resolveReference<T = DocumentData>(firestore: MockFirebaseFirestore, root: MockDocumentReference | null, isCollectionPath: boolean, path: string, startParity?: boolean, rootCollection?: MockCollectionReference<T>): MockCollectionReference<T> | MockDocumentReference<T>;
/**
 * Find the first index matching field value -pair match in a document array
 *
 * @export
 * @param {MockDocumentReference[]} docs
 * @param {((string | FieldPath)[])} fieldNames
 * @param {any[]} fieldValues
 * @returns {number}
 */
export declare function findIndexForDocument<T = DocumentData>(docs: MockDocumentReference<T>[], fieldNames: Array<string | FieldPath>, fieldValues: any[]): number;
/**
 * Test for field value name match
 *
 * @export
 * @param {MockDocumentReference} doc Document containing fields and values to be compared
 * @param {((string | FieldPath)[])} fieldNames Field names or paths to names
 * @param {any[]} fieldValues Counter pair value for field pointed at fieldNames list with the same index
 * @returns {boolean} True if all name-value -pairs match
 */
export declare function matchFields<T = DocumentData>(doc: MockDocumentReference<T>, fieldNames: Array<string | FieldPath>, fieldValues: any[]): boolean;
export declare function getFieldValue<T = DocumentData>(doc: MockDocumentReference<T>, fieldName: string | FieldPath): any;
export declare function getFieldValueFromData(data: any, fieldName: string | FieldPath): any;
export declare function validateFieldPathName(fieldName: string | FieldPath): string | FieldPath;
/**
 * Parse firestore update or set function input args back to an solid array to be used with
 * @param dataOrField
 * @param moreFieldsAndValues
 */
export declare function parseFieldValuePairsFromArgs(prefix: any[], moreFieldsAndValues: any[]): any[];
/**
 * Alter an object with a set of field value pairs
 *
 * @param firestore current firestore instance
 * @param data current document's data object
 * @param fieldValuePairs an arrau with fieldName / fieldPath - value pairs to be set to object
 * @returns altered object containing fieldValues assigned with possible FieldValue's processed
 */
export declare const setFieldValuePairs: (firestore: MockFirebaseFirestore, data: any, fieldValuePairs: any[]) => any;
