"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var MockFieldValue_1 = require("../MockFieldValue");
/**
 * Copy properties from source to target (recursively allows extension
 * of Objects and Arrays).  Scalar values in the target are over-written.
 * If target is undefined, an object of the appropriate type will be created
 * (and returned).
 *
 * We recursively copy all child properties of plain Objects in the source- so
 * that namespace- like dictionaries are merged.
 *
 * Note that the target can be a function, in which case the properties in
 * the source Object are copied onto it as static properties of the Function.
 */
// tslint:disable-next-line: cognitive-complexity
function processAndDeepMerge(firestore, target, source) {
    if (!source || !(source instanceof Object)) {
        return source;
    }
    var result;
    switch (source.constructor) {
        // case Date:
        //   // Treat Dates like scalars; if the target date object had any child
        //   // properties - they will be lost!
        //   const dateValue = source as Date;
        //   return new Date(dateValue.getTime());
        case Object:
            result = target !== undefined ? __assign({}, target) : {};
            break;
        case Array:
            // Always copy the array source and overwrite the target.
            result = target !== undefined ? target.slice() : [];
            break;
        default:
            // Not a plain Object - treat it as a scalar.
            return source;
    }
    for (var prop in source) {
        if (source.hasOwnProperty(prop)) {
            var sourceValue = source[prop];
            var targetValue = target ? target[prop] : undefined;
            if (!sourceValue) {
                result[prop] = targetValue;
                continue;
            }
            switch (sourceValue.constructor) {
                case Date:
                    // Treat Dates like scalars; if the target date object had any child
                    // properties - they will be lost!
                    var dateValue = source;
                    result[prop] = new Date(dateValue.getTime());
                    break;
                case Object:
                    result[prop] = processAndDeepMerge(firestore, targetValue, sourceValue);
                    break;
                case Array:
                    // Always copy the array source and overwrite the target.
                    result[prop] = processAndDeepMerge(firestore, targetValue ? targetValue.slice() : [], sourceValue);
                    break;
                case MockFieldValue_1.MockFieldValue:
                    MockFieldValue_1.processFieldValue(firestore, source, result, prop, sourceValue);
                    break;
                default:
                    // Not a plain Object - treat it as a scalar.
                    result[prop] = sourceValue;
            }
        }
    }
    return result;
}
exports.processAndDeepMerge = processAndDeepMerge;
function shallowCopy(obj) {
    if (!(obj && typeof obj === 'object')) {
        throw new Error('shallowCopy() expects object parameter.');
    }
    var result = {};
    for (var key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
            result[key] = obj[key];
        }
    }
    return result;
}
exports.shallowCopy = shallowCopy;
//# sourceMappingURL=manipulation.js.map