"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _1 = require(".");
var NotImplementedYet_1 = require("../utils/NotImplementedYet");
function createFirestoreMatchRuleFunction(fieldPath, opStr, value) {
    if (typeof fieldPath === 'string') {
        return function (doc) {
            if (!doc.data) {
                return false;
            }
            var field = doc.data[fieldPath];
            if (field) {
                switch (opStr) {
                    case '==':
                        return field === value;
                    case '<':
                        return field < value;
                    case '<=':
                        return field <= value;
                    case '>':
                        return field > value;
                    case '>=':
                        return field >= value;
                    case 'array-contains': {
                        if (!Array.isArray(field)) {
                            throw new _1.MockFirebaseValidationError("Field " + fieldPath + " is not an array.");
                        }
                        return field.indexOf(value) >= 0;
                    }
                    default:
                        throw new Error("Unexpection comparization operation: " + opStr);
                }
            }
            return false;
        };
    }
    throw new NotImplementedYet_1.NotImplementedYet('createFirestoreMatchRuleFunction');
}
exports.createFirestoreMatchRuleFunction = createFirestoreMatchRuleFunction;
function filterDocumentsByRules(docs, rules) {
    if (!rules) {
        return docs.filter(function (d) { return d.data !== undefined; });
    }
    return docs.filter(function (doc) {
        // Documents that do not exists, will be removed automatically from the list
        if (!doc.data) {
            return false;
        }
        for (var _i = 0, rules_1 = rules; _i < rules_1.length; _i++) {
            var rule = rules_1[_i];
            if (!doesRuleMatch(rule, doc)) {
                return false;
            }
        }
        return true;
    });
}
exports.filterDocumentsByRules = filterDocumentsByRules;
function doesRuleMatch(rule, doc) {
    var fieldPath = rule.fieldPath, opStr = rule.opStr, value = rule.value;
    if (typeof fieldPath !== 'string') {
        throw new NotImplementedYet_1.NotImplementedYet('doesRuleMatch - field value support');
    }
    var field;
    if (fieldPath.includes('.')) {
        field = doc.data;
        var paths = fieldPath.split('.');
        for (var _i = 0, paths_1 = paths; _i < paths_1.length; _i++) {
            var path = paths_1[_i];
            field = field[path];
            if (!field) {
                return value === undefined;
            }
        }
    }
    else {
        field = doc.data[fieldPath];
    }
    if (field === undefined) {
        return false;
    }
    switch (opStr) {
        case '==':
            if (field !== value) {
                return false;
            }
            break;
        case '<':
            if (field >= value) {
                return false;
            }
            break;
        case '<=':
            if (field > value) {
                return false;
            }
            break;
        case '>':
            if (field <= value) {
                return false;
            }
            break;
        case '>=':
            if (field < value) {
                return false;
            }
            break;
        case 'array-contains': {
            if (!Array.isArray(field)) {
                throw new _1.MockFirebaseValidationError("Error: Field " + fieldPath + " is not an array.");
            }
            if (field.indexOf(value) < 0) {
                return false;
            }
            break;
        }
        case 'in': {
            if (!Array.isArray(value)) {
                throw new _1.MockFirebaseValidationError("Error: Value " + value + " is not an array.");
            }
            if (value.indexOf(field) < 0) {
                return false;
            }
            break;
        }
        default:
            throw new Error("Unidentified where operation: " + opStr);
    }
    return true;
}
//# sourceMappingURL=matching.js.map