import { FieldPath, OrderByDirection, DocumentData } from '@firebase/firestore-types';
import { MockTimestamp } from '../../app';
import { MockBlob } from '../MockBlob';
import MockDocumentReference from '../MockDocumentReference';
import { MockGeoPoint } from '../MockGeoPoint';
import { MockQueryOrderRule } from '../MockQuery';
export declare type SortFunction = (a: any, b: any) => number;
/**
 * Data types supported by Firestore. Datatypes are defined in an order to match the sorting order
 * of different data types
 */
export declare enum MockFirestoreDatatype {
    UNDEFINED = 0,
    Null = 1,
    Boolean = 2,
    Integer = 3,
    Float = 4,
    Numerical = 5,
    Date = 6,
    Date_Timestamp = 7,
    Text = 8,
    Byte = 9,
    Reference = 10,
    GeoPoint = 11,
    Array = 12,
    Map = 13
}
/**
 * Generetate sorting function based on given parameters.
 *
 * @param fieldPath Name of a single field or a path object pointing to field to be used for ordering
 * @param directionStr {'asc' | 'desc' | undefined} Sorting direction. Default is `'asc'`.
 */
export declare function querySortFunction(fieldPath: string | FieldPath, directionStr?: OrderByDirection): SortFunction;
export declare function sortDocumentsByRules<T = DocumentData>(docs: MockDocumentReference<T>[], rules?: MockQueryOrderRule[]): MockDocumentReference<T>[];
export declare function getFirestoreDataType(value: any): MockFirestoreDatatype;
/**
 * Compare to objects based on Firestore's sorting rules.
 *
 * @see https://cloud.google.com/firestore/docs/concepts/data-types
 */
export declare function compareObjects(a: any, b: any): number;
export declare function compareArrays(a: any[], b: any[]): number;
export declare function compareTimestamps(a: MockTimestamp, b: MockTimestamp): number;
export declare function compareGeoPoints(a: MockGeoPoint, b: MockGeoPoint): number;
export declare function compareBlobs(a: MockBlob, b: MockBlob): number;
